==> zeebe-feel-integration-8.5.3-sources.jar: io/camunda/zeebe/feel/impl/FeelToMessagePackTransformer.scala <==
/*
 * Copyright Camunda Services GmbH and/or licensed to Camunda Services GmbH under
 * one or more contributor license agreements. See the NOTICE file distributed
 * with this work for additional information regarding copyright ownership.
 * Licensed under the Zeebe Community License 1.1. You may not use this file
 * except in compliance with the Zeebe Community License 1.1.
 */
package io.camunda.zeebe.feel.impl

import io.camunda.zeebe.feel.impl.Loggers.LOGGER
import io.camunda.zeebe.msgpack.spec.MsgPackWriter
import org.agrona.concurrent.UnsafeBuffer
import org.agrona.{DirectBuffer, ExpandableArrayBuffer}
import org.camunda.feel.syntaxtree._

class FeelToMessagePackTransformer {

  private val writer = new MsgPackWriter

  private val writeBuffer = new ExpandableArrayBuffer
  private val resultView = new UnsafeBuffer
  private val stringWrapper = new UnsafeBuffer

  def toMessagePack(value: Val): DirectBuffer = {
    writer.wrap(writeBuffer, 0)

    writeValue(value)

    resultView.wrap(writeBuffer, 0, writer.getOffset)
    resultView
  }

  private def writeValue(value: Val) {
    value match {
      case ValNull => writer.writeNil()
      case ValNumber(number) if number.isWhole => writer.writeInteger(number.longValue)
      case ValNumber(number) => writer.writeFloat(number.doubleValue)
      case ValBoolean(boolean) => writer.writeBoolean(boolean)
      case ValString(string) => writeStringValue(string)
      case ValList(items) => {
        writer.writeArrayHeader(items.size)
        items.foreach(writeValue)
      }
      case ValContext(context: MessagePackContext) => writer.writeRaw(context.messagePackMap)
      case ValContext(context) => {
        val variables = context.variableProvider.getVariables
        writer.writeMapHeader(variables.size)

        variables.foreach { case (key, value) =>
          stringWrapper.wrap(key.getBytes)
          writer.writeString(stringWrapper)

          value match {
            case entryVal: Val => writeValue(entryVal)
            case entryBuffer: DirectBuffer => writer.writeRaw(entryBuffer)
            case other => {
              writer.writeNil()
              LOGGER.trace("No FEEL to MessagePack transformation for '{}'. Using 'null' for context entry with key '{}'.", other, key)
            }
          }
        }
      }
      case ValTime(time) => writeStringValue(time.format)
      case ValLocalTime(time) => writeStringValue(time.toString)
      case ValDate(date) => writeStringValue(date.toString)
      case ValDateTime(dateTime) => writeStringValue(dateTime.toString)
      case ValLocalDateTime(dateTime) => writeStringValue(dateTime.toString)
      case ValDayTimeDuration(duration) => writeStringValue(duration.toString)
      case ValYearMonthDuration(duration) => writeStringValue(duration.toString)
      case other => {
        writer.writeNil()
        LOGGER.trace("No FEEL to MessagePack transformation for '{}'. Using 'null' instead.", other)
      }
    }
  }

  private def writeStringValue(string: String) = {
    stringWrapper.wrap(string.getBytes)
    writer.writeString(stringWrapper)
  }
}

==> zeebe-feel-integration-8.5.17-sources.jar: io/camunda/zeebe/feel/impl/FeelToMessagePackTransformer.scala <==
/*
 * Copyright Camunda Services GmbH and/or licensed to Camunda Services GmbH under
 * one or more contributor license agreements. See the NOTICE file distributed
 * with this work for additional information regarding copyright ownership.
 * Licensed under the Zeebe Community License 1.1. You may not use this file
 * except in compliance with the Zeebe Community License 1.1.
 */
package io.camunda.zeebe.feel.impl

import io.camunda.zeebe.feel.impl.Loggers.LOGGER
import io.camunda.zeebe.msgpack.spec.MsgPackWriter
import org.agrona.concurrent.UnsafeBuffer
import org.agrona.{DirectBuffer, ExpandableArrayBuffer}
import org.camunda.feel.syntaxtree._

class FeelToMessagePackTransformer {

  private val writer = new MsgPackWriter

  private val writeBuffer = new ExpandableArrayBuffer
  private val resultView = new UnsafeBuffer
  private val stringWrapper = new UnsafeBuffer

  def toMessagePack(value: Val): DirectBuffer = {
    writer.wrap(writeBuffer, 0)

    writeValue(value)

    resultView.wrap(writeBuffer, 0, writer.getOffset)
    resultView
  }

  private def writeValue(value: Val) {
    value match {
      case ValNull => writer.writeNil()
      case ValNumber(number) if number.isWhole => writer.writeInteger(number.longValue)
      case ValNumber(number) => writer.writeFloat(number.doubleValue)
      case ValBoolean(boolean) => writer.writeBoolean(boolean)
      case ValString(string) => writeStringValue(string)
      case ValList(items) => {
        writer.writeArrayHeader(items.size)
        items.foreach(writeValue)
      }
      case ValContext(context: MessagePackContext) => writer.writeRaw(context.messagePackMap)
      case ValContext(context) => {
        val variables = context.variableProvider.getVariables
        writer.writeMapHeader(variables.size)

        variables.foreach { case (key, value) =>
          stringWrapper.wrap(key.getBytes)
          writer.writeString(stringWrapper)

          value match {
            case entryVal: Val => writeValue(entryVal)
            case entryBuffer: DirectBuffer => writer.writeRaw(entryBuffer)
            case other => {
              writer.writeNil()
              LOGGER.trace("No FEEL to MessagePack transformation for '{}'. Using 'null' for context entry with key '{}'.", other, key)
            }
          }
        }
      }
      case ValTime(time) => writeStringValue(time.format)
      case ValLocalTime(time) => writeStringValue(time.toString)
      case ValDate(date) => writeStringValue(date.toString)
      case ValDateTime(dateTime) => writeStringValue(dateTime.toString)
      case ValLocalDateTime(dateTime) => writeStringValue(dateTime.toString)
      case duration: ValDayTimeDuration => writeStringValue(duration.toString)
      case duration: ValYearMonthDuration => writeStringValue(duration.toString)
      case other => {
        writer.writeNil()
        LOGGER.trace("No FEEL to MessagePack transformation for '{}'. Using 'null' instead.", other)
      }
    }
  }

  private def writeStringValue(string: String) = {
    stringWrapper.wrap(string.getBytes)
    writer.writeString(stringWrapper)
  }
}
